@@ -1,9 +1,13 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+/* This file gets installed, so we can't assume config.h is available */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <glib-object.h>
 #include <gobject/gvaluecollector.h>
-#include <cairo-gobject.h>
 
 #include "regress.h"
 
@@ -285,6 +289,7 @@ regress_test_value_return(int i)
 /************************************************************************/
 /* foreign structs */
 
+#ifndef _GI_DISABLE_CAIRO
 /**
  * regress_test_cairo_context_full_return:
  *
@@ -365,6 +370,7 @@ regress_test_cairo_surface_full_out (cairo_surface_t **surface)
 {
   *surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 10, 10);
 }
+#endif
 
 /**
  * regress_test_gvariant_i:
@@ -907,7 +913,7 @@ regress_test_array_int_null_out (int **arr, int *len)
 /* GList */
 
 static /*const*/ GList *
-regress_test_sequence_list()
+regress_test_sequence_list (void)
 {
     static GList *list = NULL;
     if (!list) {
@@ -1028,7 +1034,7 @@ regress_test_glist_null_out (GSList **out_list)
 /* GSList */
 
 static /*const*/ GSList *
-regress_test_sequence_slist()
+regress_test_sequence_slist (void)
 {
     static GSList *list = NULL;
     if (!list) {
@@ -1152,7 +1158,7 @@ static char *table_data[3][2] = {
 };
 
 static GHashTable *
-regress_test_table_ghash_new_container()
+regress_test_table_ghash_new_container (void)
 {
   GHashTable *hash;
   int i;
@@ -1163,7 +1169,7 @@ regress_test_table_ghash_new_container()
 }
 
 static GHashTable *
-regress_test_table_ghash_new_full()
+regress_test_table_ghash_new_full (void)
 {
   GHashTable *hash;
   int i;
@@ -1176,7 +1182,7 @@ regress_test_table_ghash_new_full()
 }
 
 static /*const*/ GHashTable *
-regress_test_table_ghash_const()
+regress_test_table_ghash_const (void)
 {
   static GHashTable *hash = NULL;
   if (!hash) {
@@ -1871,6 +1877,16 @@ regress_test_boxed_equals (RegressTestBoxed *boxed,
 	  regress_test_simple_boxed_a_equals(&other->nested_a, &boxed->nested_a));
 }
 
+void
+regress_test_boxeds_not_a_method (RegressTestBoxed *boxed)
+{
+}
+
+void
+regress_test_boxeds_not_a_static (void)
+{
+}
+
 static void
 regress_test_boxed_free (RegressTestBoxed *boxed)
 {
@@ -1953,6 +1969,53 @@ G_DEFINE_BOXED_TYPE(RegressTestBoxedC,
                     regress_test_boxed_c_ref,
                     regress_test_boxed_c_unref);
 
+struct _RegressTestBoxedD {
+  char *a_string;
+  gint a_int;
+};
+
+RegressTestBoxedD *
+regress_test_boxed_d_new (const char *a_string, int a_int)
+{
+  RegressTestBoxedD *boxed;
+
+  boxed = g_slice_new (RegressTestBoxedD);
+  boxed->a_string = g_strdup (a_string);
+  boxed->a_int = a_int;
+
+  return boxed;
+}
+
+RegressTestBoxedD *
+regress_test_boxed_d_copy (RegressTestBoxedD *boxed)
+{
+  RegressTestBoxedD *ret;
+
+  ret = g_slice_new (RegressTestBoxedD);
+  ret->a_string = g_strdup (boxed->a_string);
+  ret->a_int = boxed->a_int;
+
+  return ret;
+}
+
+void
+regress_test_boxed_d_free (RegressTestBoxedD *boxed)
+{
+  g_free (boxed->a_string);
+  g_slice_free (RegressTestBoxedD, boxed);
+}
+
+int
+regress_test_boxed_d_get_magic (RegressTestBoxedD *boxed)
+{
+  return strlen (boxed->a_string) + boxed->a_int;
+}
+
+G_DEFINE_BOXED_TYPE(RegressTestBoxedD,
+                    regress_test_boxed_d,
+                    regress_test_boxed_d_copy,
+                    regress_test_boxed_d_free);
+
 G_DEFINE_TYPE(RegressTestObj, regress_test_obj, G_TYPE_OBJECT);
 
 enum
@@ -1961,6 +2024,7 @@ enum
   PROP_TEST_OBJ_BOXED,
   PROP_TEST_OBJ_HASH_TABLE,
   PROP_TEST_OBJ_LIST,
+  PROP_TEST_OBJ_PPTRARRAY,
   PROP_TEST_OBJ_HASH_TABLE_OLD,
   PROP_TEST_OBJ_LIST_OLD,
   PROP_TEST_OBJ_INT,
@@ -2125,6 +2189,7 @@ regress_test_obj_default_matrix (RegressTestObj *obj, const char *somestr)
 
 enum {
   REGRESS_TEST_OBJ_SIGNAL_SIG_NEW_WITH_ARRAY_PROP,
+  REGRESS_TEST_OBJ_SIGNAL_SIG_NEW_WITH_ARRAY_LEN_PROP,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_HASH_PROP,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_STRV,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_OBJ,
@@ -2134,6 +2199,7 @@ enum {
   REGRESS_TEST_OBJ_SIGNAL_ALL,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_INT64_PROP,
   REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_UINT64_PROP,
+  REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_INTARRAY_RET,
   N_REGRESS_TEST_OBJ_SIGNALS
 };
 
@@ -2191,6 +2257,27 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
 		  1,
 		  G_TYPE_ARRAY);
 
+  /**
+   * RegressTestObj::sig-with-array-len-prop:
+   * @self: an object
+   * @arr: (array length=len) (element-type uint) (allow-none): numbers, or %NULL
+   * @len: length of @arr, or 0
+   *
+   * This test signal similar to GSettings::change-event
+   */
+  regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_SIG_NEW_WITH_ARRAY_LEN_PROP] =
+    g_signal_new ("sig-with-array-len-prop",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  0,
+		  NULL,
+		  NULL,
+		  NULL,
+		  G_TYPE_NONE,
+		  2,
+		  G_TYPE_POINTER,
+		  G_TYPE_INT);
+
   /**
    * RegressTestObj::sig-with-hash-prop:
    * @self: an object
@@ -2250,6 +2337,7 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
 		  1,
 		  G_TYPE_OBJECT);
 
+#ifndef _GI_DISABLE_CAIRO
    /**
    * RegressTestObj::sig-with-foreign-struct:
    * @self: an object
@@ -2266,6 +2354,7 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
 		  G_TYPE_NONE,
 		  1,
 		  CAIRO_GOBJECT_TYPE_CONTEXT);
+#endif
 
   regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_FIRST] =
     g_signal_new ("first",
@@ -2340,6 +2429,25 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
 		  1,
 		  G_TYPE_UINT64);
 
+  /**
+   * RegressTestObj::sig-with-intarray-ret:
+   * @self: an object
+   * @i: an integer
+   *
+   * Returns: (array zero-terminated=1) (element-type gint) (transfer full):
+   */
+  regress_test_obj_signals[REGRESS_TEST_OBJ_SIGNAL_SIG_WITH_INTARRAY_RET] =
+    g_signal_new ("sig-with-intarray-ret",
+		  G_TYPE_FROM_CLASS (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  0,
+		  NULL,
+		  NULL,
+		  g_cclosure_marshal_VOID__BOXED,
+		  G_TYPE_ARRAY,
+		  1,
+		  G_TYPE_INT);
+
   gobject_class->set_property = regress_test_obj_set_property;
   gobject_class->get_property = regress_test_obj_get_property;
   gobject_class->dispose = regress_test_obj_dispose;
@@ -2363,10 +2471,7 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
                                    pspec);
 
   /**
-   * RegressTestObj:hash-table:
-   *
-   * Type: GLib.HashTable(utf8,gint8)
-   * Transfer: container
+   * RegressTestObj:hash-table: (type GLib.HashTable(utf8,gint8)) (transfer container)
    */
   pspec = g_param_spec_boxed ("hash-table",
                               "GHashTable property",
@@ -2378,10 +2483,7 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
                                    pspec);
 
   /**
-   * RegressTestObj:list:
-   *
-   * Type: GLib.List(utf8)
-   * Transfer: none
+   * RegressTestObj:list: (type GLib.List(utf8)) (transfer none)
    */
   pspec = g_param_spec_pointer ("list",
                                 "GList property",
@@ -2392,10 +2494,18 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
                                    pspec);
 
   /**
-   * RegressTestObj:hash-table-old:
-   *
-   * Type: GLib.HashTable<utf8,gint8>
-   * Transfer: container
+   * RegressTestObj:pptrarray: (type GLib.PtrArray(utf8)) (transfer none)
+   */
+  pspec = g_param_spec_pointer ("pptrarray",
+                                "PtrArray property as a pointer",
+                                "Test annotating with GLib.PtrArray",
+                                G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_TEST_OBJ_PPTRARRAY,
+                                   pspec);
+
+  /**
+   * RegressTestObj:hash-table-old: (type GLib.HashTable<utf8,gint8>) (transfer container)
    */
   pspec = g_param_spec_boxed ("hash-table-old",
                               "GHashTable property with <>",
@@ -2407,10 +2517,7 @@ regress_test_obj_class_init (RegressTestObjClass *klass)
                                    pspec);
 
   /**
-   * RegressTestObj:list-old:
-   *
-   * Type: GLib.List<utf8>
-   * Transfer: none
+   * RegressTestObj:list-old: (type GLib.List<utf8>) (transfer none)
    */
   pspec = g_param_spec_pointer ("list-old",
                                 "GList property with ()",
@@ -2553,6 +2660,7 @@ regress_test_obj_emit_sig_with_obj (RegressTestObj *obj)
     g_object_unref (obj_param);
 }
 
+#ifndef _GI_DISABLE_CAIRO
 void
 regress_test_obj_emit_sig_with_foreign_struct (RegressTestObj *obj)
 {
@@ -2560,6 +2668,7 @@ regress_test_obj_emit_sig_with_foreign_struct (RegressTestObj *obj)
   g_signal_emit_by_name (obj, "sig-with-foreign-struct", cr);
   cairo_destroy (cr);
 }
+#endif
 
 void
 regress_test_obj_emit_sig_with_int64 (RegressTestObj *obj)
@@ -2587,6 +2696,17 @@ regress_test_obj_instance_method (RegressTestObj *obj)
     return -1;
 }
 
+/**
+ * regress_test_obj_instance_method_full:
+ * @obj: (transfer full):
+ *
+ */
+void
+regress_test_obj_instance_method_full (RegressTestObj *obj)
+{
+  g_object_unref (obj);
+}
+
 double
 regress_test_obj_static_method (int x)
 {
@@ -2829,15 +2949,13 @@ regress_test_obj_skip_inout_param (RegressTestObj *obj,
 }
 
 /**
- * regress_test_obj_do_matrix:
+ * regress_test_obj_do_matrix: (virtual matrix)
  * @obj: A #RegressTestObj
  * @somestr: Meaningless string
  *
  * This method is virtual.  Notably its name differs from the virtual
  * slot name, which makes it useful for testing bindings handle this
  * case.
- *
- * Virtual: matrix
  */
 int
 regress_test_obj_do_matrix (RegressTestObj *obj, const char *somestr)
@@ -3087,12 +3205,7 @@ regress_test_fundamental_object_init (GTypeInstance * instance, gpointer klass)
 }
 
 /**
- * RegressTestFundamentalObject:
- *
- * Ref Func: regress_test_fundamental_object_ref
- * Unref Func: regress_test_fundamental_object_unref
- * Set Value Func: regress_test_value_set_fundamental_object
- * Get Value Func: regress_test_value_get_fundamental_object
+ * RegressTestFundamentalObject: (ref-func regress_test_fundamental_object_ref) (unref-func regress_test_fundamental_object_unref) (set-value-func regress_test_value_set_fundamental_object) (get-value-func regress_test_value_get_fundamental_object)
  */
 
 GType
@@ -3266,6 +3379,37 @@ int regress_test_array_callback (RegressTestCallbackArray callback)
   return sum;
 }
 
+/**
+ * regress_test_array_inout_callback:
+ * @callback: (scope call):
+ *
+ */
+int
+regress_test_array_inout_callback (RegressTestCallbackArrayInOut callback)
+{
+  int *ints;
+  int length;
+
+  ints = g_new (int, 5);
+  for (length = 0; length < 5; ++length)
+    ints[length] = length - 2;
+
+  callback (&ints, &length);
+
+  g_assert_cmpint (length, ==, 4);
+  for (length = 0; length < 4; ++length)
+    g_assert_cmpint (ints[length], ==, length - 1);
+
+  callback (&ints, &length);
+
+  g_assert_cmpint (length, ==, 3);
+  for (length = 0; length < 3; ++length)
+    g_assert_cmpint (ints[length], ==, length);
+
+  g_free (ints);
+  return length;
+}
+
 /**
  * regress_test_simple_callback:
  * @callback: (scope call) (allow-none):
@@ -3514,6 +3658,18 @@ regress_test_owned_gerror_callback (RegressTestCallbackOwnedGError callback)
   callback (error);
 }
 
+/**
+ * regress_test_skip_unannotated_callback: (skip)
+ * @callback: No annotation here
+ *
+ * Should not emit a warning:
+ * https://bugzilla.gnome.org/show_bug.cgi?id=685399
+ */
+void
+regress_test_skip_unannotated_callback (RegressTestCallback callback)
+{
+}
+
 /* interface */
 
 static void
@@ -3747,6 +3903,7 @@ regress_test_torture_signature_2 (int                   x,
   *y = x;
   *z = x * 2;
   *q = g_utf8_strlen (foo, -1) + m;
+  callback(user_data);
   notify (user_data);
 }
 
@@ -3893,11 +4050,22 @@ regress_has_parameter_named_attrs (int        foo,
 /**
  * regress_test_versioning:
  *
- * Since: 1.32.1
- * Deprecated: 1.33.3: Use foobar instead
- * Stability: Unstable
+ * Since: 1.32.1: Actually, this function was introduced earlier
+ *   than this, but it didn't do anything before this version.
+ * Deprecated: 1.33.3: This function has been deprecated,
+ *   because it sucks. Use foobar instead.
+ * Stability: Unstable: Maybe someday we will find the time
+ *   to stabilize this function. Who knows?
  */
 void
 regress_test_versioning (void)
 {
 }
+
+void
+regress_like_xkl_config_item_set_name (RegressLikeXklConfigItem *self,
+                                       const char *name)
+{
+  strncpy (self->name, name, sizeof (self->name) - 1);
+  self->name[sizeof(self->name)-1] = '\0';
+}
